import numpy as np
import plotly.express as px
import plotly.graph_objects as go
from typing import Iterable, Optional

from chemeye.arrays import tsne


class TSNEPlotter:
    def __init__(self, descriptors:np.array) -> None:
        self.__descriptors = np.copy(descriptors)
        
    def plot(self, x_name, y_name, color_category:Optional[Iterable]=None) -> go.Figure:
        arr = tsne(self.__descriptors)
        df = ({
            x_name: arr[:, 0],
            y_name: arr[:, 1]
        })

        opacity = 1
        color = None
        if color_category:
            df['color'] = color_category
            opacity = 0.25
            color = 'color'
            
        return px.scatter(df, x=x_name, y=y_name, color=color, render_mode='svg', opacity=opacity)



from typing import Iterable
import plotly.graph_objects as go
import plotly.express as px

from chemeye.arrays import sim_matrix
from chemeye.__asset_loader import default_simmat_options as default_options


class SimMat:
    def __init__(self, row_prints:Iterable, col_prints:Iterable, options:dict=default_options, 
                 key_type:str='ecfp') -> None:
        self.options = options
        sim_arr = sim_matrix(row_prints=row_prints, col_prints=col_prints, key_type=key_type)
        self.fig = px.imshow(sim_arr, color_continuous_scale=options['color_scale'])
        self.update()
        
    def __title(self) -> None:
        self.fig.update_layout(title_text=self.options['title']['text'])
        
        title_pos = self.options['title']['position']
        self.fig.update_layout(title_xanchor=title_pos['x'], title_yanchor=title_pos['y'])
        
    def __axes(self) -> None:
        x = self.options['axes']['x']
        y = self.options['axes']['y']
        
        self.fig.update_xaxes(visible=x['show'])
        self.fig.update_yaxes(visible=y['show'])
        
        self.fig.update_layout(xaxis_side=x['position'], yaxis_side=y['position'])
        
    def update(self) -> go.Figure:
        self.__title()
        self.__axes()
        return self.fig



import unittest
import naclo
from chemeye import SimMat
from plotly.graph_objects import Figure


class TestSimMat(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.test_ecfp = naclo.mols_2_ecfp(
            naclo.smiles_2_mols([
                'CCC',
                'O',
                'C',
                'SOO'
            ]))
        return super().setUpClass()
    
    def test_init(self):
        sim_mat = SimMat(self.test_ecfp, self.test_ecfp)
        self.assertIsInstance(
            sim_mat.fig,
            Figure
        )



import numpy as np
from rdkit import DataStructs
from typing import Iterable
from sklearn.manifold import TSNE


def sim_matrix(row_prints:Iterable, col_prints:Iterable, key_type:str='ecfp') -> np.array:
    full_array = np.zeros((len(row_prints), len(col_prints)))

    for i, row in enumerate(row_prints):
        for j, col in enumerate(col_prints):
            if key_type == 'ecfp':
                similarity = DataStructs.TanimotoSimilarity(row, col)
            elif key_type == 'maccs':
                similarity = DataStructs.FingerprintSimilarity(row, col)
            else:
                raise('Key type must be either "ecfp" or "maccs".')
                
            full_array[i][j] = similarity
    
    return full_array

def tsne(descriptors:np.array, seed=42) -> np.array:
    tsne = TSNE(n_components=2, random_state=seed, learning_rate='auto', init='pca')
    return tsne.fit_transform(descriptors)



from setuptools import setup, find_packages


setup(
    name='chemeye',
    version='0.0.3',
    license='MIT',
    author='Jacob Gerlach',
    author_email='jwgerlach00@gmail.com',
    url='https://github.com/jwgerlach00/chemeye',
    description='Visualization toolset for small molecule drug discovery',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    package_data={'chemeye': ['assets/default_simmat_options.json']},
    python_requires='>=3.6',
    install_requires=[
        'numpy',
        'plotly',
        'rdkit',
        'rdkit_pypi',
        'scikit-learn'
    ],
)



import unittest
import plotly.graph_objects as go

import naclo
from chemeye import TSNEPlotter


class TestTSNEPlotter(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.test_smiles = ['CN=C=O', 'CCC', 'O']
        cls.test_mols = naclo.smiles_2_mols(cls.test_smiles)
        cls.maccs_keys = naclo.mols_2_maccs(cls.test_mols)
        cls.ecfp4_prints = naclo.mols_2_ecfp(cls.test_mols, radius=2, return_numpy=True)
        
        cls.tsne_plotter = TSNEPlotter(cls.ecfp4_prints)
        return super().setUpClass()
    
    def test_plot(self):
        fig = self.tsne_plotter.plot('tsne1', 'tsne2', [1, 'a', True])
        
        self.assertIsInstance(
            fig,
            go.Figure
        )
